<?php
namespace Fiscal\Logic\ImportaRelacProdItGen;

use Fiscal\Logic\ImportaTipoItem\TipoItem;
use Fiscal\Logic\ImportaGeneroProd\GeneroProduto;
use Geral\Util\DateUtil;
use Geral\Util\Ambiente;
use Indeterminado\Entities\RelacProdItGen;
use Setforms2\Shared\Registry;

class RelacProdItemGenero
{
    public static function replicaProdItGen($nrorg, $cdLivroOrig, $cdLivroDest) {
        
        /*Validação para verificar se os cadastros de tipo item/gênero e produto do livro de origem estão vazios.*/
        TipoItem::validaTipoItemLivroVazio($nrorg, $cdLivroOrig);
        GeneroProduto::validaGeneroProdLivroVazio($nrorg, $cdLivroOrig);
        self::validaProdItGenLivroVazio($nrorg, $cdLivroOrig);

        $connection = \Setforms2\Shared\Registry::getEntityManagerConnection();
        $connection->beginTransaction();

        try {
            self::replicaRegistrosProdItGen($nrorg, $cdLivroOrig, $cdLivroDest);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function validaProdItGenLivroVazio($nrorg, $cdLivroMag)
    {
        /*A função valida se existe algum item/gênero por produto cadastrado para o livro informado como parãmetro.*/
        $proditgen = RelacProdItGen::findByNrorgAndCdlivromag($nrorg, $cdLivroMag);

        if ($proditgen == null) {
            throw new \Exception ("Operação Bloqueada! Não existe cadastro de Tipo de Item/Gênero por Produto para o livro magnético de origem informado.");
        }
    }

    public static function replicaRegistrosProdItGen($nrorg, $cdLivroOrig, $cdLivroDest)
    {
        /*A função replica os tipo item/gênero por produto entre os livros conforme os parâmetros informados, de forma que,
          são excluídos todos os produtos do livro de destino e então são inseridos todos os cadastros do livro
          de origem para o livro de destino.*/

        try {

            /*São apagados os cadastros do livro de destino para serem incluídos os novos.*/
            $repositorioRelacproditgen = \Indeterminado\Entities\Relacproditgen::getRepository();
            $repositorioRelacproditgen->deleteByCdlivromagAndNrorg($cdLivroDest, $nrorg);
            Registry::getEntityManager()->flush();

            /*A rotina deve substituir os cadastos de tipo de item e gênero do produto para que não ocorram erros de pk ao inserir os produtos.*/
            TipoItem::replicaRegistrosItem($nrorg, $cdLivroOrig, $cdLivroDest);
            GeneroProduto::replicaRegistrosGenero($nrorg, $cdLivroOrig, $cdLivroDest);

            /*São buscados todos os item/gênero por produto do livro de origem e, um a um, têm seu código do livro
              alterado para o código do livro de destino.*/
            $repositorioRelacproditgen->replicaItensPorLivro($cdLivroOrig, $cdLivroDest, $nrorg);
            Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }

    }

}